function [] = Aufgabe3
dbstop if error

% Input
f = @(x) Inhalt(x);
gradf = @(x) Gradient_Inhalt(x);
% Powell-Wolfe Parameter
gamma = 0.01;
eta = 0.5;
% Anfangs/Randwert: die Funktion sin(xy\pi)
n = 20; % Anzahl Knoten pro Richtung
h = 1/(n -1); % Schrittweite
% Gitterpunkte erstellen
x = 0:h:1;
y = 0:h:1;
[X,Y] = meshgrid(x,y);
% Funktion auswerten
Z = sin(Y.*X.*pi);
U0 = Z(:);
tol = 1e-06; % Toleranz für beide Verfahren - hier geringer!


% lege die skalierung der Identität fest
B0 = speye(length(U0)); %oder
% B0 = speye(length(U0))/20;

% Quasi-Newton-verfahren getestet - U_it als zusätzlicher output
[AU,U_it] = Glob_Quasi_Newton_BFGS(f,gradf,U0,B0,gamma,eta,tol);
fprintf('Mit dem Globalisierten BFGS-Verfahren finden wir das Minimum A(U) = %f in %f Schritten.\n',AU(end),length(AU))

% plot
figure
for q = 1:size(U_it,1)
   plot_results(U_it(q,:));
end




function [A,U_it] = Glob_Quasi_Newton_BFGS(I,gradI,U0,B0,gamma,eta,tol)

% ersten Schritt speichern
i = 1;
U_it(i,:) = U0;
A(i,1) = I(U_it(i,:)); 
% erste Aufdatierung - Startmatrix
B = B0;

while (true)
    
    if norm(gradI(U_it(i,:))) < tol % Abbruchsbedingung
        return
    else
        % Gehe mindestens einen Schritt
        dk = -B*gradI(U_it(i,:)); % Berechne nur die Suchrichtung. Die Schrittweite ist immer = 1 - Spaltenvektor
        %keyboard;
        % Schrittweite mit Powell-Wolfe bestimmen
        s = Powell_Wolfe(I,gradI,U_it(i,:)',dk,gamma,eta);
        %s = 1;
        i = i+1; % In den nächsten Schritt gehen
        U_it(i,:) = U_it(i-1,:) + s*dk'; % Schritt vollziehen - nun mit Schrittweite!
        A(i,1) = I(U_it(i,:)); % Funktionswerte speichern - optional
    end
    
    disp([norm(gradI(U_it(i,:))),A(i,1),s]) % to track how long it will take until termination.... optional!
    if norm(gradI(U_it(i,:))) < tol % nochmal Abbruchsbedingung, da im letzten Schritt die Aufdatierung unnötig ist
        return
    else % falls nicht abgebrochen wird -> Aufdatierung
        yk = gradI(U_it(i,:)) - gradI(U_it(i-1,:)); % Vorarbeit für die Aufdatierung - Spaltenvektor
        % Aufdatierung berechnen (mit den transponiert aufpassen)
        B = B + ((dk - B*yk)*dk'+dk*(dk-B*yk)')./(dk'*yk) - ((dk-B*yk)'*yk*(dk*dk'))./(dk'*yk*dk'*yk);
    end
end


function s = Powell_Wolfe(f,gradf,z,direction,gamma,eta)
% Realisierung der Powell-Wolfe-Schrittweitenregel

% s_minus halbieren, bis es die Armijo-Bedingung erfüllt
s_minus = 2;
while (true)
    s_minus = s_minus/2;
    if f(z+s_minus*direction) - f(z) <= gamma*s_minus*gradf(z)*direction'; % Armijo-Bedingung
        break
    end
end

% s_plus verdoppeln, bis es die Armijo-Bedingung verletzt
s_plus = s_minus;
while (true)
    s_plus = 2*s_plus;
    if f(z+s_plus*direction) - f(z) > gamma*s_plus*gradf(z)*direction'; % negierte Armijo-Bedingung
        break
    end
end

% Intervallhalbierung bis die Powell-Wolfe-Bedingung erfüllt ist
while gradf(z+s_minus*direction)*direction' < eta*gradf(z)*direction'
    s_star = (s_plus + s_minus)/2; % Intervallmitte
    % Falls Intervallmitte Armijo erfüllt, rechte Intervallhälfte wählen
    if f(z+s_star*direction) - f(z) <= gamma*s_star*gradf(z)*direction';
        s_minus = s_star;
    else % andernfalls linke Intervallhälfte wählen
        s_plus = s_star;
    end
end
s = s_minus;