function gradient = Gradient_Inhalt(U)
% Funktion welche zu einem gegebenen Vektor von z-Koordinaten des
% Minimalflächenproblems den Gradienten rechnet. Dabei ist die übliche
% Diskretisierung schon angenommen

n = sqrt(length(U)); % Anzahl Knoten in jede Richtung
if floor(n) ~= n
    error('Anzahl Knoten ist nicht quadratisch!')
end
N = n^2; % Anzahl Knoten
%wandel anzahl Knoten in Schrittweite um, da diese im Gradient vorkommt
h = 1/(n -1);

% nicht vektorisierte Version - und daher sehr langsam!
% gradient = zeros(N,1);
% % aber der Gradient ist nur in den inneren Knoten nicht null!
% for i = 1:n-2
%     for j = 1:n-2
%         Ind = i*n+j+1; % Index des aktuellen inneren Punktes in der gesamten Diskretisierung
%         % die Indizes der 6 beteiligten Knoten (involviert Randknoten) der
%         % 6 beteiligten Dreiecke
%         Ind_SW = Ind-n-1;
%         Ind_S = Ind-n;
%         Ind_W = Ind -1;
%         Ind_O = Ind+1;
%         Ind_N = Ind+n;
%         Ind_NO = Ind+n+1;
%         % Damit die einzelnen Terme, welche den Gradienten formen
%         Term_SW = (U(Ind) - U(Ind_S))/sqrt((U(Ind_S) - U(Ind_SW))^2 + (U(Ind_S) - U(Ind))^2 + h^2); %süd_westliches Dreieck
%         Term_WS = (U(Ind) - U(Ind_W))/sqrt((U(Ind) - U(Ind_W))^2 + (U(Ind_W) - U(Ind_SW))^2 + h^2); %west_südliches Dreieck
%         Term_SO = (2*U(Ind) - U(Ind_O) - U(Ind_S))/sqrt((U(Ind_O) - U(Ind))^2 + (U(Ind) - U(Ind_S))^2 + h^2); %süd_östliches Dreieck
%         Term_WN = (2*U(Ind) - U(Ind_W) - U(Ind_N))/sqrt((U(Ind) - U(Ind_W))^2 + (U(Ind) - U(Ind_N))^2 + h^2); %west_nördliches Dreieck
%         Term_NO = (U(Ind) - U(Ind_O))/sqrt((U(Ind_O) - U(Ind))^2 + (U(Ind_O) - U(Ind_NO))^2 + h^2); %nord_östliches Dreieck
%         Term_ON = (U(Ind) - U(Ind_N))/sqrt((U(Ind_NO) - U(Ind_N))^2 + (U(Ind_N) - U(Ind))^2 + h^2); %ost_nördliches Dreieck
%         gradient(Ind) = h/2*(Term_SW+Term_WS+Term_SO+Term_WN+Term_NO+Term_ON);
%     end
% end

% vektorisierte Version
l = (n-2)^2; %number of inner nodes
gradient = zeros(N,1);
% Die Menge der Indizes der inneren Knoten
Ind = zeros(l,1);
for i = 1:n-2
    for j = 1:n-2
        Ind((i-1)*(n-2) + j) = i*n + j+1;
    end
end
% Dementsprechen die Indizes der beteiligten Knoten pro Innerer Knoten
Ind_SW = Ind-n-1;
Ind_S = Ind-n;
Ind_W = Ind -1;
Ind_O = Ind+1;
Ind_N = Ind+n;
Ind_NO = Ind+n+1;
% Damit die verschiedenen Terme der beteiligten Dreiecke
Term_SW = (U(Ind) - U(Ind_S))./sqrt((U(Ind_S) - U(Ind_SW)).^2 + (U(Ind_S) - U(Ind)).^2 + h^2); %süd_westliches Dreieck
Term_WS = (U(Ind) - U(Ind_W))./sqrt((U(Ind) - U(Ind_W)).^2 + (U(Ind_W) - U(Ind_SW)).^2 + h^2); %west_südliches Dreieck
Term_SO = (2*U(Ind) - U(Ind_O) - U(Ind_S))./sqrt((U(Ind_O) - U(Ind)).^2 + (U(Ind) - U(Ind_S)).^2 + h^2); %süd_östliches Dreieck
Term_WN = (2*U(Ind) - U(Ind_W) - U(Ind_N))./sqrt((U(Ind) - U(Ind_W)).^2 + (U(Ind) - U(Ind_N)).^2 + h^2); %west_nördliches Dreieck
Term_NO = (U(Ind) - U(Ind_O))./sqrt((U(Ind_O) - U(Ind)).^2 + (U(Ind_O) - U(Ind_NO)).^2 + h^2); %nord_östliches Dreieck
Term_ON = (U(Ind) - U(Ind_N))./sqrt((U(Ind_NO) - U(Ind_N)).^2 + (U(Ind_N) - U(Ind)).^2 + h^2); %ost_nördliches Dreieck
% Als Summe dann der gradient
gradient(Ind) = h/2*(Term_SW+Term_WS+Term_SO+Term_WN+Term_NO+Term_ON);
