function A = Inhalt(U)
% Funktion, welche den gewollten Flächeninhalt des Graphen berechnet als
% Summe der Inhalte der Graphen jedes Dreiecks.

n = sqrt(length(U)); % Anzahl Knoten in jede Richtung
if floor(n) ~= n
    error('Anzahl Knoten ist nicht quadratisch!')
end
N = n^2; % Anzahl Knoten
anzT = 2*(n-1)^2; % Anzahl Dreiecke
%wandel anzahl Knoten in Schrittweite um, um die Koordinaten der
%Knoten x_1, ... , x_H zu erhalten
h = 1/(n -1);

% Erstelle eine Matrix mit Ortsvektoren der Punkte (x_i(1), x_i(2), z(i)).
% Damit wird später der Flächeninhalt über das Kreuzprodukt gerechnet.
Koords = zeros(N,3);
for i = 1:n
    for j = 1:n
        Koords((i-1)*n + j,:) = [(j-1)*h, (i-1)*h, U((i-1)*n + j)];
    end
end

% Erstelle eine Matrix von Elementindizes, welche angibt, welche Knoten
% welches Triangulierungselement bilden.
Elementind = zeros(anzT,3);
for i = 1:n-1
    for j = 1:n-1
        ind = (i-1)*n + j; % index des linken unteren Knotens
        Elementind((ind-i)*2+1,:) = [ind, ind+1, ind+n+1];
        Elementind((ind-i)*2+2,:) = [ind, ind+n, ind+n+1];
    end
end

% nicht vektorisierte und daher langsame Version!
% Berechne nun die Fläche als Summe über den Flächeninhalt aller Dreiecke.
% Dabei gehören zu jedem Dreieck 3 Koordinaten k1, k2, k3 aus Koords. 
% Nun ist der Flächeninhalt 1/2*||(k2 - k1) x (k3 - k1)||.
% A = 0; % initialisieren
% for i = 1:anzT
%     k1 = Koords(Elementind(i,1),:);
%     k2 = Koords(Elementind(i,2),:);
%     k3 = Koords(Elementind(i,3),:);
%     A = A+ 0.5*norm(cross(k2-k1, k3-k1));
% end

% Hier die vektorisierte Version:
% in jeder Zeile stehen die 1. bzw. 2./3. Koordinate des entsprechenden
% Dreiecks (1 Zeile pro Dreieck)
k1 = Koords(Elementind(:,1),:);
k2 = Koords(Elementind(:,2),:);
k3 = Koords(Elementind(:,3),:);
% die differenzvektoren
vec1 = k2 - k1;
vec2 = k3 - k1;
% nun das vektorisierte Kreuzprodukt
crossprod = cross(vec1,vec2,2);
% nun Schrittweise die euklidische Norm
crossprod_squared = crossprod.^2;
crossprod_summed = sum(crossprod_squared,2);
crossprod_root = sqrt(crossprod_summed);
sum_crossprods = sum(crossprod_root);
% noch*1/2, vgl. Formel
A = 1/2*sum_crossprods;